.include "tn2313def.inc"								;  - ATTiny2313

.equ hex = 16											; -  
.equ vr_addr = 0b00000000
.equ vl_addr = 0b00000001
.equ vt_addr = 0b00000011
.equ vb_addr = 0b00000010
.equ st_addr = 0b00000101
.equ fd_addr = 0b00000100

;********* Init_I2C *************************			 I2C
;**** Global I2C Constants *********
.equ	SCLP	= 5										; SCL Pin number (port D)
.equ	SDAP	= 4										; SDA Pin number (port D)
.equ	b_dir	= 0										; transfer direction bit in i2cadr
.equ	i2crd	= 1
.equ	i2cwr	= 0
;**** Global Register Variables ****
.def	i2cdelay= r22									; Delay loop variable
.def	i2cdata	= r23									; I2C data transfer register
.def	i2cadr	= r24									; I2C address and direction register
.def	i2cstat	= r27									; I2C bus status register
;============================================

;********* Init_RC5 *************************			 RC5
;00110000 - TV											;   
;10110000 - Mute
;01100100 - Sleep
;00001000 - Volume up
;10001000 - Volume down
;00000100 - Channel up
;10000100 - Channel down
;============================================
.equ	INPUT	=0					    				;PD0
.equ	SYS_ADDR=0										;   ( . =0)
.def	S		=R0
.def	inttemp	=R1
.def	ref1	=R2
.def	ref2	=R3
.def	temp	=R16
.def	timerL	=R17
.def	timerH	=R16
.def	system	=R19
.def	command	=R20									;,      
.def	bitcnt	=R21
;============================================
.def	data		=r29
.def	addr		=r25
.def	lcd_count	=r24
.def	vol_both	=r9
.def	vol_right	=r8
.def	vol_left	=r7
.def	vtreble		=r6
.def	vbass		=r5
.def	stat		=r4

.org    0											
rjmp reset									;    

;********* Init_Timer ***********************  
;============================================
.org OVF0addr
TIM0_OVF:	
		in	S,sreg
		inc	timerL					
		inc	inttemp
		brne	TIM0_OVF_exit
		inc	timerH
;============================================
TIM0_OVF_exit:
		out	sreg,S
		reti
;============================================

;********* Init_LCD *************************  
;============================================		
initlcd:
	rcall ddelay
	rcall ddelay
	ldi addr,$30 		; 
	rcall icom_2313 		
	rcall delay
	ldi addr,$30 		; 
	rcall icom_2313 		
	rcall delay
	ldi addr,$30 		; 
	rcall icom_2313 		
	rcall ddelay

	ldi	addr,0b00101000 	;4- , 2- 
	rcall icom_2313 			;, -58 

	ldi addr,0b00001100		; ,  
	rcall icom_2313
		
	ldi addr,0b00000110 	;  ,   
	rcall icom_2313		

	ldi addr,0b00000010 	;      DDRAM, AC = 0	      
	rcall icom_2313

ret							;

;============================================

;********* Init_main ************************  
;============================================
reset:
	ldi temp,low(RAMEND)					 ; 
	out	SPL,temp			
	ldi	temp,1								 ;/ 0   
	out	TCCR0,temp
	ldi	temp,1<<TOIE0						 ;    
	out	TIMSK,temp
;============================================
    ser data            				 	 ;  B  
    out ddrb,data
  	ldi data,0b01111110	
    out ddrd,data 						     ;  D   (1,2,3,6 )
 	ldi data,0b01001110						 ;  D   (1,2,3,6 )
    out portd,data
    rcall initlcd							 ;   
	
sei	
;============================================
;     

	ldi addr,0b00000000
	rcall i2c_rd
	rcall delay

	ldi data,0b00101111
	ldi addr,fd_addr
	rcall i2c_trance


	ldi addr,st_addr
	rcall i2c_rd
	mov stat,i2cdata
	rcall delay
	ldi temp,0b01111000
	or stat,temp

	ldi addr,vb_addr
	rcall i2c_rd
	mov vbass,i2cdata
	rcall delay

	ldi addr,vt_addr
	rcall i2c_rd
	mov vtreble,i2cdata
	rcall delay

	ldi temp,0b11110000
	or vtreble,temp
	or vbass,temp

	ldi addr,vl_addr
	rcall i2c_rd
	mov vol_left,i2cdata
	rcall delay

	ldi addr,vr_addr
	rcall i2c_rd
	mov vol_right,i2cdata

	ldi temp,0b11000000
	or vol_right,temp
	or vol_left,temp
;============================================

;********* Main *****************************		  
;============================================
	rcall clean				; 
	
	ldi r30,low(l_intro1*2)	;  Z    l_intro1
    ldi r31,high(l_intro1*2);  1   
    ldi lcd_count,16 
	rcall ddram
	
	ldi addr,0b11000000      ;   DDRAM    
    rcall icom_2313	
	ldi lcd_count,16 
	ldi r30,low(l_intro2*2)  ;  Z    l_intro2
    ldi r31,high(l_intro2*2) ;  2  
    rcall ddram

	rcall ddelay
;============================================  
volume_both:								; 

	rcall clean				; 
	ldi r30,low(l_volume*2) ;  
    ldi r31,high(l_volume*2)				
    ldi lcd_count,10 
	rcall ddram

	rcall delay				; 
							
volume_both_nc:
							;     
	rcall clean_2
 
	cp vol_right,vol_left 
	brsh dt
	rjmp dt1	

dt: mov temp,vol_right
	rjmp cont

dt1:mov temp,vol_left

cont:	
	andi temp,0b00111111
	cpi temp,0b000000100
	brlo volume_both_l
	asr temp
	asr temp
	brcc nmax_vol
	inc temp
nmax_vol:
	mov lcd_count,temp

	ldi r30,low(l_bar*2)    					
    ldi r31,high(l_bar*2)
	rcall ddram

///////////////////////////
volume_both_l:
	rcall scan								;
		
	cpi command,0b00010000
	breq vb_up
	cpi command,0b00010001
	breq vb_down

	cpi command,0b00001100
	breq to_treble
    cpi command,0b00100110
	breq vb_sw_c
    cpi command,0b00001101
	breq vb_mute

rjmp volume_both_l

vb_sw_c:
	rcall sw_channel
	rjmp volume_both
vb_mute:	
	rcall sw_mute
	rjmp volume_both

to_treble:
	rcall vol_mem
rjmp treble

///////////////////////////
vb_up:	
	ldi temp,0b11111111
	cp vol_right,temp
	breq vb_trance
	cp vol_left,temp
	breq vb_trance

	inc vol_right
	inc vol_left
	rjmp vb_trance

vb_down:
	ldi temp,0b11000000
	cp vol_right,temp
	breq vb_trance	
	cp vol_left,temp
	breq vb_trance

	dec vol_right
	dec vol_left
	rjmp vb_trance

vb_trance:								;   
	rcall vol_audio

rjmp volume_both_nc
;============================================
wtembr:
	ldi r30,low(l_tembr*2) ;  " "
    ldi r31,high(l_tembr*2)				
    ldi lcd_count,6 
	rcall ddram
ret
;============================================
treble:						; 

	rcall clean				; 
	rcall wtembr			;
	
	ldi r30,low(l_treble*2) ;  
    ldi r31,high(l_treble*2)				
    ldi lcd_count,7 
	rcall ddram
	rcall delay				; 
							
treble_nc:
							;     
	rcall clean_2
 
	mov temp,vtreble
	andi temp,0b00001111
	cpi temp,0
	breq treble_l
	mov lcd_count,temp
	ldi r30,low(l_bar*2)   
    ldi r31,high(l_bar*2)
	rcall ddram

///////////////////////////
treble_l:
	rcall scan	
									;
	cpi command,0b00010000
	breq t_up
	cpi command,0b00010001
	breq t_down

	cpi command,0b00001100
	breq to_bass
    cpi command,0b00100110
	breq t_sw_c
    cpi command,0b00001101
	breq t_mute

rjmp treble_l

t_sw_c:
	rcall sw_channel
	rjmp treble
t_mute:	
	rcall sw_mute
	rjmp treble

to_bass:
	ldi addr,vt_addr
	mov data,vtreble
	rcall i2c_wr
rjmp bass

///////////////////////////
t_up:	
	ldi temp,0b11111111
	cp vtreble,temp
	breq t_trance
	inc vtreble
	rjmp t_trance

t_down:
	ldi temp,0b11110000
	cp vtreble,temp
	breq t_trance	
	dec vtreble
	rjmp t_trance

t_trance:				;   
	mov data,vtreble
	ldi addr,vt_addr
	rcall i2c_trance

rjmp treble_nc
;============================================
bass:								; h 

	rcall clean				; 
	rcall wtembr

	ldi r30,low(l_bass*2) ;  
    ldi r31,high(l_bass*2)				
    ldi lcd_count,7 
	rcall ddram

	rcall delay				; 
							
bass_nc:
							;     
	rcall clean_2
 
	mov temp,vbass
	andi temp,0b00001111
	cpi temp,0
	breq bass_l
	mov lcd_count,temp
	ldi r30,low(l_bar*2)    				
    ldi r31,high(l_bar*2)
	rcall ddram

///////////////////////////
bass_l:
	rcall scan								;
	
	cpi command,0b00010000
	breq b_up
	cpi command,0b00010001
	breq b_down

	cpi command,0b00001100
	breq to_balance
    cpi command,0b00100110
	breq b_sw_c
    cpi command,0b00001101
	breq b_mute

rjmp bass_l

b_sw_c:
	rcall sw_channel
	rjmp bass
b_mute:	
	rcall sw_mute
	rjmp bass

to_balance:
	ldi addr,vb_addr
	mov data,vbass
	rcall i2c_wr

rjmp balance

///////////////////////////
b_up:	
	ldi temp,0b11111111
	cp vbass,temp
	breq b_trance
	inc vbass
	rjmp b_trance

b_down:
	ldi temp,0b11110000
	cp vbass,temp
	breq b_trance	
	dec vbass
	rjmp b_trance

b_trance:									;   
	mov data,vbass
	ldi addr,vb_addr
	rcall i2c_trance

rjmp bass_nc
;============================================
balance:
	rcall clean								; 

	ldi r30,low(l_balance*2)    			;  
    ldi r31,high(l_balance*2)				
    ldi lcd_count,6 
	rcall ddram

	rcall delay

balance_nc:

	rcall clean_2
 
 	ldi temp,7

	mov data,vol_right
	andi data,0b00111111
	lsr data
	lsr data
	lsr data
	
	add temp,data
	
	mov data,vol_left
	andi data,0b00111111
	lsr data
	lsr data
	lsr data
	
	sub temp,data
	
	cpi temp,0b00000000
	breq abs_left	

	mov lcd_count,temp
	ldi r30,low(l_cln*2)    		
    ldi r31,high(l_cln*2)
	rcall ddram
	
	cp vol_right,vol_left
	breq abs_eq
abs_left:
	ldi addr,$ff
	rcall idat_2313
	rcall idat_2313

rjmp balance_l

abs_eq:
	ldi addr,$3d
	rcall idat_2313
	rcall idat_2313

balance_l:	
	rcall scan
												
	cpi command,0b00010000
	breq b_right
	cpi command,0b00010001
	breq b_left

	cpi command,0b00001100
	breq to_volume_both
    cpi command,0b00100110
	breq bal_sw_c
    cpi command,0b00001101
	breq bal_mute
	
rjmp balance_l

bal_sw_c:
	rcall sw_channel
rjmp balance

bal_mute:	
	rcall sw_mute
	rjmp balance

to_volume_both:
	rcall vol_mem
rjmp volume_both

///////////////////////////
b_right:	
	ldi temp,0b11000000
	cp vol_left,temp
	breq bal_trance
	ldi temp,0b11111111
	cp vol_right,temp
	breq bal_trance
	
	dec vol_left
	inc vol_right
	rjmp bal_trance
b_left:
	ldi temp,0b11000000
	cp vol_right,temp
	breq bal_trance
	ldi temp,0b11111111
	cp vol_left,temp
	breq bal_trance

	dec vol_right
	inc vol_left
	rjmp bal_trance
bal_trance:	
	rcall vol_audio

rjmp balance_nc
;============================================     
vol_mem:
	ldi addr,vr_addr
	mov data,vol_right
	rcall i2c_wr
	rcall delay	
	ldi addr,vl_addr
	mov data,vol_left
	rcall i2c_wr
ret
;============================================    
vol_audio:
	mov data,vol_right
	ldi addr,vr_addr
	rcall i2c_trance
	
	mov data,vol_left	
	ldi addr,vl_addr
	rcall i2c_trance
ret
;============================================
sw_channel:	
	rcall clean
													; DDRAM
    ldi r30,low(l_channel*2)    					;  Z    l_intro
    ldi r31,high(l_channel*2)						;  
    ldi lcd_count,6 
	rcall ddram
	
	bst stat,0
	brts to2
	bst stat,1
	brts to3
	bst stat,2
	brts to1

	rjmp to1
to3:
	ldi temp,0b00000100
	or stat,temp

	ldi temp,0b11111100	
	and stat,temp

	ldi addr,$33					;3
	rcall idat_2313
	
	rjmp swc_end
to2:
	ldi temp,0b00000010
	or stat,temp
	
	ldi temp,0b11111010	
	and stat,temp

	ldi addr,$32					;2
	rcall idat_2313
	
	rjmp swc_end
to1:
	ldi temp,0b00000001
	or stat,temp

	ldi temp,0b11111001	
	and stat,temp

	ldi addr,$31					;1
	rcall idat_2313

	rjmp swc_end
swc_end:
	ldi addr,st_addr
	mov data,stat
	rcall i2c_trance

	ldi addr,st_addr
	mov data,stat
	rcall i2c_wr

	rcall ddelay
	
	ret											;  
;============================================
sw_mute:	
	mov vol_left,vol_right
							
	rcall clean
													; DDRAM
    ldi r30,low(l_mute*2)    					
    ldi r31,high(l_mute*2)					
    ldi lcd_count,6 
	rcall ddram
	
	bst stat,7
	brtc mute_off
	rjmp mute_on
mute_on:
	ldi temp,0b01111111
	and stat,temp	
    ldi r30,low(l_on*2)    						
    ldi r31,high(l_on*2)					
    ldi lcd_count,6 
	rcall ddram

	rjmp swc_end
mute_off:
	ldi temp,0b10000000
	or stat,temp
    ldi r30,low(l_off*2)    					
    ldi r31,high(l_off*2)						
    ldi lcd_count,7 
	rcall ddram

	rjmp swc_end
											;
;============================================
clean:												;   
	ldi addr,0b10000000      				
    rcall icom_2313 
	ldi lcd_count,16
	ldi r30,low(l_cln*2)    						
    ldi r31,high(l_cln*2)			
	rcall ddram
	ldi addr,0b10000000      				
    rcall icom_2313 	
	ret
;============================================
clean_2:
	ldi addr,0b11000000           					;      DDRAM
    rcall icom_2313									;   40
	ldi lcd_count,16
	ldi r30,low(l_cln*2)    					
    ldi r31,high(l_cln*2)
	rcall ddram
	ldi addr,0b11000000           					;      DDRAM
    rcall icom_2313	
	ret
;********* RC5 ******************************		       
;============================================		  RC5
detect:	clr	inttemp
		clr	timerH
detect1:clr	timerL
detect2:cpi	timerH,8					;If line not idle within 131ms
		brlo	dl1
		rjmp	fault					;then exit
dl1:	cpi	timerL,55					;If line low for 3.5ms
		brge	start1					;then wait for start bit
		sbis	PIND,INPUT				;If line is
		rjmp	detect1					;low  - jump to detect1
		rjmp	detect2					;high - jump to detect2
start1:	cpi	timerH,8					;If no start bit detected
		brge	fault					;within 130ms then exit
		sbic	PIND,INPUT				;Wait for start bit
		rjmp	start1
		clr	timerL					;Measure length of start bit
start2:	cpi	timerL,17					;If startbit longer than 1.1ms,
		brge	fault					;exit
		sbis	PIND,INPUT
		rjmp	start2
								;Positive edge of 1st start bit
		mov	temp,timerL				;timer is 1/2 bit time
		clr	timerL
		mov	ref1,temp
		lsr	ref1
		mov	ref2,ref1
		add	ref1,temp				;ref1 = 3/4 bit time
		lsl	temp
		add	ref2,temp				;ref2 = 5/4 bit time
start3:	cp	timerL,ref1					;If high periode St2 &gt; 3/4 bit time
		brge	fault					;exit
		sbic	PIND,INPUT				;Wait for falling edge start bit 2
		rjmp	start3
		clr	timerL
		ldi	bitcnt,12					;Receive 12 bits
		clr	command
		clr	system
sample:	cp	timerL,ref1					;Sample INPUT at 1/4 bit time
		brlo	sample
		sbic	PIND,INPUT	
		rjmp	bit_is_a_1				;Jump if line high
bit_is_a_0:	clc						;Store a '0'
		rol	command
		rol	system
								;Synchronize timing
bit_is_a_0a:	cp	timerL,ref2				;If no edge within 3/4 bit time
		brge	fault					;exit
		sbis	PIND,INPUT				;Wait for rising edge
		rjmp	bit_is_a_0a				;in the middle of the bit
		clr	timerL
		rjmp	nextbit
bit_is_a_1:	sec						;Store a '1'
		rol	command
		rol	system
 	 							;Synchronize timing
bit_is_a_1a:cp	timerL,ref2					;If no edge within 3/4 bit time
		brge	fault					;exit
		sbic	PIND,INPUT				;Wait for falling edge
		rjmp	bit_is_a_1a				;in the middle of the bit
		clr	timerL
nextbit:dec	bitcnt						;If bitcnt &gt; 0
		brne	sample					;get next bit
								;All bits sucessfully received!
		mov	temp,command				;Place system bits in "system"
		rol	temp		
		rol	system
		rol	temp
		rol	system
		bst	system,5					;Move toggle bit
		bld	command,6				;to "command"
								;Clear remaining bits
		andi	command,0b01111111
		andi	system,0x1F
		ret
fault:	ser	command					;Both "command" and "system"
		ser	system					;0xFF indicates failure
		ret
;============================================

;********* Scan *****************************
;============================================
scan:	
	ldi command,0
	sbis pind,1
	ldi command,0b00100110 
	sbis pind,2
	ldi command,0b00010000  
	sbis pind,3
	ldi command,0b00010001 
	sbis pind,6
	ldi command,0b00001100 
	cpi command,0
	breq scan_rc5
	rcall delay
ret
;============================================
scan_rc5:
	rcall detect
	andi command,0x3F					;  
	cpi command,0x3f
	breq scan
ret	
;============================================
;********* PAUSES ***************************
;============================================
ddelay:				;	 
        ldi r19,10
dd:		rcall delay
        dec r19
        brne dd
        ret	
del:	
        ldi r18,50            						;  
l4:     dec r18                 			
        brne l4                 			
        ret
delay:
        ldi r17,50             						; 
l:      ldi r18,255             			
l1:     dec r18                 
        brne l1                 
        dec r17                 
        brne l                  
        ret
;============================================	

;********* LCD ******************************
;============================================	
ddram:	
cli
	lpm		                     						;  flash-  	
    mov addr,r0
    rcall idat_2313        								;   DDRAM
    rcall del
    ld r1,z+               								;  1  Z  flash-
    dec lcd_count                 							;     
    brne ddram  
sei	            							
    ret       		; 
;============================================
;RS - 4,  E - 5, 0 - AD4, 1 - AD5, 2 - AD6, 3 - AD7
icom_2313:
    swap addr               							;    
    mov r26,addr
    andi r26,$ef            							;  4  D ( RS=0)
    ori  r26,$e0            							;  5  D (  E=1)

	out portb,r26	
    cbi portb, 5            							;   5  D (  E=0). 
    swap addr                							;    
    mov r26,addr
    andi r26,$ef            							;  4  D ( RS=0)	
    ori  r26,$e0            							;  5  D (  E=1)

    out portb,r26
    cbi portb, 5            							;   5  D (  E=0)
	rcall del
    ret
;
idat_2313:
    swap addr  			     							;    
    mov r26,addr	
    ori  r26,$f0        	 		   					;  4  D ( RS=1) 

	out portb,r26           							; 5  D (  E=1)
    cbi portb,5           								;   5  D (  E=0)
    swap addr             	 		   			 		;    
    mov r26,addr		
    ori  r26,$f0       	 	 							;  4  D ( RS=1) 

    out portb,r26      		   							; 5  D (  E=1)
    cbi portb,5             							;   5  D (  E=0)
	rcall del		
    ret
;============================================

;********* I2C ******************************
;============================================	
i2c_trance:

	ldi	i2cadr,0b10000000;$A0+i2cwr	;Set device address and write
	rcall	i2c_start				;Send start condition and address

	mov	i2cdata,addr				;Write word address (0x00)
	rcall	i2c_do_transfer			;Execute transfer

	mov	i2cdata,data					;Set write data to 01010101b
	rcall	i2c_do_transfer			;Execute transfer
	rcall	i2c_stop				;Send stop condition
ret
;============================================	
i2c_rd:

	ldi		i2cadr,0b10100000	; Set device address and write
	rcall 	i2c_start			; Send start condition and address

	mov		i2cdata,addr		; Write word address
	rcall 	i2c_do_transfer		; Execute transfer

	ldi		i2cadr,0b10100001	; Set device address and read
	rcall	i2c_rep_start		; Send repeated start condition and address

	sec							; Set no acknowledge (read is followed by a stop condition)
	rcall	i2c_do_transfer		; Execute transfer (read)

	rcall	i2c_stop	
ret
;============================================	
i2c_wr:

	ldi		i2cadr,0b10100000	; Set device address and write
	rcall 	i2c_start			; Send start condition and address

	mov		i2cdata,addr		; Write word address
	rcall 	i2c_do_transfer		; Execute transfer

	mov		i2cdata,data		; Set write data to temp
	rcall	i2c_do_transfer		; Execute transfer

	rcall	i2c_stop		; Send stop condition
ret
;============================================	
i2c_hp_delay:
	ldi	i2cdelay,2
i2c_hp_delay_loop:
	dec	i2cdelay
	brne	i2c_hp_delay_loop
	ret
i2c_qp_delay:
	ldi	i2cdelay,1	
i2c_qp_delay_loop:
	dec	i2cdelay
	brne	i2c_qp_delay_loop
	ret
i2c_rep_start:
	sbi	DDRD,SCLP				; force SCL low
	cbi	DDRD,SDAP				; release SDA
	rcall	i2c_hp_delay		; half period delay
	cbi	DDRD,SCLP				; release SCL
	rcall	i2c_qp_delay		; quarter period delay
i2c_start:				
	mov	i2cdata,i2cadr			; copy address to transmitt register
	sbi	DDRD,SDAP				; force SDA low
	rcall	i2c_qp_delay		; quarter period delay
i2c_write:
	sec							; set carry flag
	rol	i2cdata					; shift in carry and out bit one
	rjmp	i2c_write_first
i2c_write_bit:
	lsl	i2cdata					; if transmit register empty
i2c_write_first:
	breq	i2c_get_ack			; goto get acknowledge
	sbi	DDRD,SCLP				; force SCL low
	brcc	i2c_write_low		; if bit high
	nop							; (equalize number of cycles)
	cbi	DDRD,SDAP				; release SDA
	rjmp	i2c_write_high
i2c_write_low:					; else
	sbi	DDRD,SDAP				; force SDA low
	rjmp	i2c_write_high		; (equalize number of cycles)
i2c_write_high:
	rcall	i2c_hp_delay		; half period delay
	cbi	DDRD,SCLP				; release SCL
	rcall	i2c_hp_delay		; half period delay
	rjmp	i2c_write_bit
i2c_get_ack:
	sbi	DDRD,SCLP				; force SCL low
	cbi	DDRD,SDAP				; release SDA
	rcall	i2c_hp_delay		; half period delay
	cbi	DDRD,SCLP				; release SCL
i2c_get_ack_wait:
	sbis	PIND,SCLP			; wait SCL high 
								;(In case wait states are inserted)
	rjmp	i2c_get_ack_wait
	clc							; clear carry flag
	sbic	PIND,SDAP			; if SDA is high
	sec							; set carry flag
	rcall	i2c_hp_delay		; half period delay
	ret
i2c_do_transfer:
	sbrs	i2cadr,b_dir		; if dir = write
	rjmp	i2c_write			;goto write data
i2c_read:
	rol	i2cstat					; store acknowledge
								; (used by i2c_put_ack)
	ldi	i2cdata,0x01			; data = 0x01
i2c_read_bit:					; do
	sbi	DDRD,SCLP				; force SCL low
	rcall	i2c_hp_delay		; half period delay
	cbi	DDRD,SCLP				; release SCL
	rcall	i2c_hp_delay		; half period delay
	clc							; clear carry flag
	sbic	PIND,SDAP			; if SDA is high
	sec							; set carry flag
	rol	i2cdata					; store data bit
	brcc	i2c_read_bit		; while receive register not full
i2c_put_ack:
	sbi	DDRD,SCLP				; force SCL low
	ror	i2cstat					; get status bit
	brcc	i2c_put_ack_low		; if bit low goto assert low
	cbi	DDRD,SDAP				; release SDA
	rjmp	i2c_put_ack_high
i2c_put_ack_low:				; else
	sbi	DDRD,SDAP				; force SDA low
i2c_put_ack_high:
	rcall	i2c_hp_delay		; half period delay
	cbi	DDRD,SCLP				; release SCL
i2c_put_ack_wait:
	sbis	PIND,SCLP			; wait SCL high
	rjmp	i2c_put_ack_wait
	rcall	i2c_hp_delay		; half period delay
	ret
i2c_stop:
	sbi	DDRD,SCLP				; force SCL low
	sbi	DDRD,SDAP				; force SDA low
	rcall	i2c_hp_delay		; half period delay
	cbi	DDRD,SCLP				; release SCL
	rcall	i2c_qp_delay		; quarter period delay
	cbi	DDRD,SDAP				; release SDA
	rcall	i2c_hp_delay		; half period delay
	ret
i2c_init:

	ldi i2cstat,0b01111110
	out	DDRD,i2cstat

	clr	i2cstat				
	out	PORTD,i2cstat		; set I2C pins to open colector


	ret
;============================================	

;********* Constant_LCD**********************			  (    )
;============================================
.org $300
l_cln:		.DB ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '
			.DB ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '
			.DB ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '
			.DB ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '
			.DB ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' '

l_bar:		.DB $ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff,$ff

l_intro1:	.DB '"','H','O','B','A',$b1,' ','T','E','X','H',$a5,'K','A','"',' '
l_intro2:	.DB 'C','T','E','P','E','O','T','E','M',$a0,'P','O',$a0,$a7,'O','K'
			
l_volume:	.DB $a1,'P','O','M','K','O','C','T','b',' ',' ',' ',' ',' ',' ',' '
l_tembr	:	.DB 'T','E','M',$a0,'P',' ',' ',' '
l_bass	:	.DB 'H',$a5,$a4,'K',$a5,'E',' ',' '
l_treble:	.DB 'B',$ae,'C','O','K',$a5,'E',' '
l_mute	:	.DB $a4,'B',$a9,'K',' ','B',' ',' '
l_on	:	.DB 'K',$a7,$b0,$ab,'E','H',' ',' '
l_off	:	.DB $ae,'K',$a7,$b0,$ab,'E','H',' '
l_balance:  .DB $a0,'A',$a7,'A','H','C',' ',' '
l_channel:  .DB 'K','A','H','A',$a7,' ',' ',' '

